﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using static System.Windows.Forms.VisualStyles.VisualStyleElement;

using OpcLabs.BaseLib.ComponentModel;
using OpcLabs.EasyOpc.DataAccess;
using OpcLabs.EasyOpc.DataAccess.OperationModel;

namespace ExcelOPC64
{
    public partial class FormMain : Form
    {
        String OPCMachineName = "127.0.0.1";
        String OPCServerClass = "Kepware.KEPServerEX.V6";

        private OpcLabs.EasyOpc.DataAccess.EasyDAClient easyDAClient;
        private int OPC_UPDATE_RATE = 1000;

        private string tagValue;
        enum STATUS
        {
            OFF = 0,
            ON = 1,
        }

        private STATUS conStatus = STATUS.OFF;
        public FormMain()
        {
            InitializeComponent();
        }

        private void FormMain_Load(object sender, EventArgs e)
        {
            easyDAClient = new EasyDAClient();
            //easyDAClient.InstanceParameters.EnableNativeClient = false;
            //easyDAClient.InstanceParameters.EnableNetApiClient = false;
            //easyDAClient.InstanceParameters.EnableServiceClient = false;


            var eventHandler = new EasyDAItemChangedEventHandler(EasyDAClient_ItemChanged);
            easyDAClient.ItemChanged += eventHandler;
            
            tbOPCMachineName.Text = OPCMachineName;
            tbOPCServerClass.Text = OPCServerClass;
        }

        private void EasyDAClient_ItemChanged(object sender, EasyDAItemChangedEventArgs e)
        {
            if (e.Vtq == null)
            {
                tbLog.AppendText($"Potential communication problem (ID = {e.Arguments.ItemDescriptor.ItemId})");
                tbLog.AppendText(Environment.NewLine);
                return;
            }

            try
            {
                var tagValue = e.Vtq.DisplayValue(); // Tag Value            
                var tagID = (int)e.Arguments.State; // ID

                switch (tagID)
                {
                    case 1:
                        this.tagValue = tagValue;
                        break;
                }
            }
            catch (Exception E)
            {
                HandleException("SubscribeTag()", E);
            }
        }

        private void timerGUI_Tick(object sender, EventArgs e)
        {
            tbTagValue.BeginInvoke(new System.Action(() =>
            {
                tbTagValue.Text = tagValue;
            }));
        }

        private void btOPC_Click(object sender, EventArgs e)
        {
            switch (conStatus)
            {
                case STATUS.OFF:
                    SubscribeTag();
                    if (!timerGUI.Enabled)
                        timerGUI.Start();
                    btOPC.Text = "DISCONNECT";
                    conStatus = STATUS.ON;
                    break;
                case STATUS.ON:
                    UnsubscribeTag();
                    if (timerGUI.Enabled)
                        timerGUI.Stop();
                    btOPC.Text = "CONNECT";
                    conStatus = STATUS.OFF;
                    break;
            }
        }

        private void HandleException(string header, Exception e)
        {
            tbLog.AppendText($"[{DateTime.Now}] {header.ToUpper()} FAILED.");
            tbLog.AppendText($"{Environment.NewLine}");
            tbLog.AppendText($"{e}");
            tbLog.AppendText($"{Environment.NewLine}{Environment.NewLine}");
        }
        private void HandleMessage(string header, string message)
        {
            tbLog.AppendText($"[{DateTime.Now}] {header.ToUpper()} SUCCEEDED.");
            tbLog.AppendText($"{Environment.NewLine}");
            tbLog.AppendText($"{message}");
            tbLog.AppendText($"{Environment.NewLine}{Environment.NewLine}");
        }

        private void SubscribeTag()
        {
            try
            {
                var argumentArray = new DAItemGroupArguments[1];
                argumentArray[0] = new DAItemGroupArguments(
                    tbOPCMachineName.Text,
                    tbOPCServerClass.Text,
                    tbTagName.Text,
                    OPC_UPDATE_RATE,
                    1);
                int[] itemIndexes = easyDAClient.SubscribeMultipleItems(argumentArray);
                tbLog.AppendText(($"Subscribed: {itemIndexes.Length}"));
                tbLog.AppendText($"{Environment.NewLine}");
            }
            catch (Exception E)
            {
                HandleException("SubscribeTag()", E);
            }
        }
        private void UnsubscribeTag()
        {
            try
            {
                easyDAClient.UnsubscribeAllItems();
                tbLog.AppendText(($"Unsubscribed"));
                tbLog.AppendText($"{Environment.NewLine}");
            }
            catch (Exception E)
            {
                HandleException("UnsubscribeTag()", E);
            }
        }

        private void btnExcelWrite_Click(object sender, EventArgs e)
        {
            try
            {
                double numberA = 0.0;
                double numberB = 0.0;

                using (var test = new ExcelTest(tbExcelPath.Text))
                {
                    (numberA, numberB) = test.WriteTest();
                    test.Dispose();
                }

                HandleMessage("Write test", $"B10 cell should contain number {numberA} in it. C10 cell should contain number {numberB}.");
            }
            catch (Exception E)
            {
                HandleException("Write test", E);
            }
        }

        private void btnExcelRead_Click(object sender, EventArgs e)
        {
            try
            {
                double numberA = 0.0;
                double numberB = 0.0;

                using (var test = new ExcelTest(tbExcelPath.Text))
                {
                    (numberA, numberB) = test.ReadTest();
                    test.Dispose();
                }

                HandleMessage("Read test", $"B10 cell contains number {numberA} in it. C10 cell contains number {numberB}.");
            }
            catch (Exception E)
            {
                HandleException("Read test", E);
            }
        }

        private void btnExcelCalculate_Click(object sender, EventArgs e)
        {
            try
            {
                double numberA = 0.0;
                double numberB = 0.0;

                using (var test = new ExcelTest(tbExcelPath.Text))
                {
                    (numberA, numberB) = test.CalcaulationsTest();
                    test.Dispose();
                }

                HandleMessage("Calculations test", $"B11 cell contains B10+C10 and equals {numberA}. C11 cell contains B10-C10 and equals {numberB}.");
            }
            catch (Exception E)
            {
                HandleException("Calculations test", E);
            }
        }
    }
}
